//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/FTDistributionItems.h
//! @brief     Defines FTDistribution1DItem's classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_FTDISTRIBUTIONITEMS_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_FTDISTRIBUTIONITEMS_H

#include "GUI/Model/Types/DoubleDescriptor.h"
#include "GUI/Model/Types/DoubleProperty.h"
#include "Sample/Correlations/FTDistributions1D.h"
#include "Sample/Correlations/FTDistributions2D.h"
#include <memory>

class Serializer;

class FTDistribution1DItem {
private:
    static constexpr auto P_OMEGA{"Omega"};

public:
    virtual std::unique_ptr<IFTDistribution1D> createFTDistribution() const = 0;
    virtual ~FTDistribution1DItem() = default;
    virtual void serialize(Serializer& s);

    DoubleDescriptor omega() const;

    virtual DoubleDescriptors valueDescriptors() const;

protected:
    FTDistribution1DItem();

    DoubleProperty m_omega;
};

class FTDistribution1DCauchyItem : public FTDistribution1DItem {
public:
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const override;
};

class FTDistribution1DGaussItem : public FTDistribution1DItem {
public:
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const override;
};

class FTDistribution1DGateItem : public FTDistribution1DItem {
public:
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const override;
};

class FTDistribution1DTriangleItem : public FTDistribution1DItem {
public:
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const override;
};

class FTDistribution1DCosineItem : public FTDistribution1DItem {
public:
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const override;
};

class FTDistribution1DVoigtItem : public FTDistribution1DItem {
public:
    FTDistribution1DVoigtItem();
    std::unique_ptr<IFTDistribution1D> createFTDistribution() const override;
    void serialize(Serializer& s) override;

    DoubleDescriptor eta() const;
    DoubleDescriptors valueDescriptors() const override;

private:
    DoubleProperty m_eta;
};

// --------------------------------------------------------------------------------------------- //

class FTDistribution2DItem {
public:
    virtual ~FTDistribution2DItem() = default;
    virtual std::unique_ptr<IFTDistribution2D> createFTDistribution() const = 0;
    virtual void serialize(Serializer& s);

    DoubleDescriptor omegaX() const;
    void setOmegaX(double omega_x);
    DoubleDescriptor omegaY() const;
    void setOmegaY(double omega_y);

    DoubleDescriptor gamma() const;
    void setGamma(double gamma);

    virtual DoubleDescriptors valueDescriptors() const;

protected:
    FTDistribution2DItem();

    DoubleProperty m_omegaX;
    DoubleProperty m_omegaY;
    DoubleProperty m_gamma;
};

class FTDistribution2DCauchyItem : public FTDistribution2DItem {
public:
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const override;
};

class FTDistribution2DGaussItem : public FTDistribution2DItem {
public:
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const override;
};

class FTDistribution2DGateItem : public FTDistribution2DItem {
public:
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const override;
};

class FTDistribution2DConeItem : public FTDistribution2DItem {
public:
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const override;
};

class FTDistribution2DVoigtItem : public FTDistribution2DItem {
public:
    FTDistribution2DVoigtItem();
    std::unique_ptr<IFTDistribution2D> createFTDistribution() const override;
    void serialize(Serializer& s) override;

    DoubleDescriptor eta() const;
    void setEta(double eta);

    DoubleDescriptors valueDescriptors() const override;

private:
    DoubleProperty m_eta;
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_FTDISTRIBUTIONITEMS_H
