//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Util/OSInfo.h
//! @brief     Defines namespace GUI::Util::OS
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_UTIL_OSINFO_H
#define BORNAGAIN_GUI_UTIL_OSINFO_H

#include <QString>

namespace GUI::Util::OS {

enum EOsType { WINDOWS_OS, LINUX_OS, MAC_OS, OTHER_UNIX_OS, OTHER_OS };

class HostOsInfo {
public:
    static inline EOsType hostOs();

    static bool isWindowsHost() { return hostOs() == WINDOWS_OS; }
    static bool isLinuxHost() { return hostOs() == LINUX_OS; }
    static bool isMacHost() { return hostOs() == MAC_OS; }
};

EOsType HostOsInfo::hostOs()
{
#if defined(Q_OS_WIN)
    return WINDOWS_OS;
#elif defined(Q_OS_LINUX)
    return LINUX_OS;
#elif defined(Q_OS_MAC)
    return MAC_OS;
#elif defined(Q_OS_UNIX)
    return OTHER_UNIX_OS;
#else
    return OTHER_OS;
#endif
}

} // namespace GUI::Util::OS

#endif // BORNAGAIN_GUI_UTIL_OSINFO_H
